package POWER::Event::IO::PROXY::HTTPS;

use version; $VERSION = qv('0.3.0');    # update POD & Changes & README

use warnings;
use strict;
use Carp;

use Scalar::Util qw( dualvar weaken );
use MIME::Base64;

use base qw( POWER::Event::IO::plugin );
use POWER::Event::IO::share;


sub new {
    my ($class, $opt_ref) = @_;
    croak '{host}+{port} required'
        if !defined $opt_ref->{host}
        || !defined $opt_ref->{port}
        ;
    my $self = $class->SUPER::new($opt_ref);
    $self->{host} = $opt_ref->{host};
    $self->{port} = $opt_ref->{port};
    if (defined $opt_ref->{user}) {
        croak '{pass} required' if !defined $opt_ref->{pass};
        $self->{user} = $opt_ref->{user};
        $self->{pass} = $opt_ref->{pass};
        $self->{_header_auth}
            = 'Proxy-Authorization: Basic '
            . encode_base64($self->{user}.':'.$self->{pass}, q{})
            . "\r\n"
            ;
    }
    return $self;
}

sub RESOLVED {
    my ($self, $host, $ip, $error) = @_;
    if (defined $ip) {
        $self->{ip} = $ip;
        $self->{_done}++;
        $self->_config($self->{_io}) if length $self->{out_buf};
    }
    else {
        $self->{_io}->{_events}    |= $ERR;
        $self->{_io}->{_err}        = $error;
        $Ready{ $self->{_io}->{fd} }= $self->{_io};
    }
    return;
}

sub ATTACH {
    my ($self, $io) = @_;
    $self->{_io} = $io;
    weaken $self->{_io};
    $DNS->_resolve($self->{host}, $self);
}

sub CONFIG {
    my ($self, $io) = @_;
    $self->{out_buf}
        = "CONNECT $io->{ip}:$io->{port} HTTP/1.0\r\n"
        . (defined $self->{_header_auth} ? $self->{_header_auth} : q{})
        . "\r\n"
        ;
    $io->{_write_needed} = 1;
    $self->_config($io) if defined $self->{ip};
    return;    
}

sub _config {
    my ($self, $io) = @_;
    local $io->{ip}     = $self->{ip};
    local $io->{port}   = $self->{port};
    $self->_real_config($io);
    unshift @WRITE, $io;
    weaken $WRITE[0];
    return;
}

sub READ {
    my ($self, $io) = @_;
    $self->_real_read($io);
    return 0 if $io->{_events} & $ERR;
    my $n = 0;
    if ($self->{in_buf} =~ s{\A(HTTP/\d\.\d (\d+) .*?)\r?\n\r?\n}{}s) {
        my ($reply, $status) = ($1, $2);
        if ($status != 200) {
            $io->{_events} |= $ERR;
            $io->{_err}     = dualvar(-1, 'https proxy: '.$reply);
        }
        # No need in this plugin after ANY proxy reply.
        $io->_detach($self);
        # Propagate any possible readed data after proxy reply.
        $_[2]  .= $self->{in_buf};
        $n      = length $self->{in_buf};
        # Force write cos we probably ignored some write requests in WRITE.
        unshift @WRITE, $io;
        weaken $WRITE[0];
    }
    return $n;
}

sub WRITE {
    my ($self, $io, undef, $out_pos) = @_;
    # Ignore all write requests and output only own data.
    $self->_real_write($io);
    return 0;
}


1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Event::IO::PROXY::HTTPS - https proxy plugin for POWER::Event::IO


=head1 VERSION

This document describes POWER::Event::IO::PROXY::HTTPS version 0.3.0


=head1 SYNOPSIS

See L<POWER::Event::IO/SYNOPSIS>.

=head1 DESCRIPTION

See L<POWER::Event::IO/PLUGINS>.

=head1 INTERFACE

=over

=item B<new( \%PARAMS )>

    ( POWER::Event::IO).

 %PARAMS     ,     
   :

    :
  host + port
    :
  user + pass

  ,       
. L</Public FIELDS>.

     [A-Z].    
     new()      .

  .


=back

=head1 Public FIELDS

        
      .

       B<RO>   
  B<RW> ,     .

          
 _     (     
    )    [A-Z]
  (     new() /
//   ).


=over

=item B<host>           I<RO>      new()

IP-    -.


=item B<port>           I<RO>      new()

  -.


=item B<user>           I<RO>      new()

   Basic-  -.
   ,    .


=item B<pass>           I<RO>      new()

   Basic-  -.
     user.


=back

=head1 DIAGNOSTICS

=head2 EXCEPTIONS

=over

=item C<< {host}+{port} required >>

   {host}  {port}   new().


=item C<< {pass} required >>

  {user},    {pass}   new().


=back

=head1 DEPENDENCIES

 POWER::Event::IO


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2006, Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.



